@@ -95,11 +95,13 @@ xkl_xkb_init_config_registry(XklConfigRegistry * config)
 }
 
 gboolean
-xkl_xkb_load_config_registry(XklConfigRegistry * config)
+xkl_xkb_load_config_registry(XklConfigRegistry * config,
+			     gboolean if_extras_needed)
 {
 	return xkl_config_registry_load_helper(config,
 					       XKB_DEFAULT_RULESET,
-					       XKB_BASE "/rules");
+					       XKB_BASE "/rules",
+					       if_extras_needed);
 }
 
 #ifdef LIBXKBFILE_PRESENT
@@ -176,21 +178,62 @@ xkl_xkb_config_native_cleanup(XklEngine * engine,
 	g_free(component_names_ptr->geometry);
 }
 
+static gchar *
+xkl_config_get_current_group_description(XklEngine * engine)
+{
+	XklState state;
+
+	xkl_xkb_get_server_state(engine, &state);
+
+	int group = state.group;
+	if ((group < 0) ||
+	    (group >=
+	     xkl_engine_backend(engine, XklXkb,
+				cached_desc)->ctrls->num_groups))
+		return NULL;
+
+	return g_strdup(xkl_engine_backend(engine, XklXkb, group_names)
+			[group]);
+}
+
+static void
+xkl_config_set_group_by_description(XklEngine * engine, gchar * descr)
+{
+	int group, n_groups;
+	gchar **group_names;
+
+	if (descr == NULL)
+		return;
+
+	// perhaps could be made mode lightweight?
+	xkl_engine_reset_all_info(engine, FALSE,
+				  "Direct reload on activation");
+
+	n_groups =
+	    xkl_engine_backend(engine, XklXkb,
+			       cached_desc)->ctrls->num_groups;
+	group_names = xkl_engine_backend(engine, XklXkb, group_names);
+
+	for (group = 0; group < n_groups; group++, group_names++) {
+		if (!g_ascii_strcasecmp(descr, *group_names)) {
+			xkl_debug(150,
+				  "Found the group with the same description, %d: [%s]\n",
+				  group, *group_names);
+			xkl_engine_lock_group(engine, group);
+			break;
+		}
+	}
+
+	g_free(descr);
+}
+
 static XkbDescPtr
 xkl_config_get_keyboard(XklEngine * engine,
 			XkbComponentNamesPtr component_names_ptr,
 			gboolean activate)
 {
 	XkbDescPtr xkb = NULL;
-#if 0
-	xkb = XkbGetKeyboardByName(_xklDpy,
-				   XkbUseCoreKbd,
-				   &componentNames,
-				   XkbGBN_AllComponentsMask &
-				   (~XkbGBN_GeometryMask),
-				   XkbGBN_AllComponentsMask &
-				   (~XkbGBN_GeometryMask), activate);
-#else
+
 	char xkm_fn[L_tmpnam];
 	char xkb_fn[L_tmpnam];
 	FILE *tmpxkm;
@@ -199,6 +242,9 @@ xkl_config_get_keyboard(XklEngine * engine,
 
 	Display *display = xkl_engine_get_display(engine);
 
+	gchar *preactivation_group_description = activate ?
+	    xkl_config_get_current_group_description(engine) : NULL;
+
 	if (tmpnam(xkm_fn) != NULL && tmpnam(xkb_fn) != NULL) {
 		pid_t cpid, pid;
 		int status = 0;
@@ -240,7 +286,7 @@ xkl_config_get_keyboard(XklEngine * engine,
 
 			XSync(display, False);
 			/* From this point, ALL errors should be intercepted only by libxklavier */
-			xkl_engine_priv(engine, criticalSection) = TRUE;
+			xkl_engine_priv(engine, critical_section) = TRUE;
 
 			cpid = fork();
 			switch (cpid) {
@@ -251,11 +297,11 @@ xkl_config_get_keyboard(XklEngine * engine,
 			case 0:
 				/* child */
 				xkl_debug(160, "Executing %s\n", XKBCOMP);
-				xkl_debug(160, "%s %s %s %s %s %s %s\n",
-					  XKBCOMP, XKBCOMP, "-I",
+				xkl_debug(160, "%s %s %s %s %s %s %s %s\n",
+					  XKBCOMP, XKBCOMP, "-w0", "-I",
 					  "-I" XKB_BASE, "-xkm", xkb_fn,
 					  xkm_fn);
-				execl(XKBCOMP, XKBCOMP, "-I",
+				execl(XKBCOMP, XKBCOMP, "-w0", "-I",
 				      "-I" XKB_BASE, "-xkm", xkb_fn,
 				      xkm_fn, NULL);
 				xkl_debug(0, "Could not exec %s: %d\n",
@@ -309,8 +355,7 @@ xkl_config_get_keyboard(XklEngine * engine,
 								}
 							} else	/* no activate, just load */
 								xkb =
-								    result.
-								    xkb;
+								    result.xkb;
 						} else {	/* could not load properly */
 
 							xkl_debug(0,
@@ -356,7 +401,11 @@ xkl_config_get_keyboard(XklEngine * engine,
 			}
 			XSync(display, False);
 			/* Return to normal X error processing */
-			xkl_engine_priv(engine, criticalSection) = FALSE;
+			xkl_engine_priv(engine, critical_section) = FALSE;
+
+			if (activate)
+				xkl_config_set_group_by_description(engine,
+								    preactivation_group_description);
 
 			xkl_debug(160,
 				  "Unlinking the temporary xkb file %s\n",
@@ -380,7 +429,6 @@ xkl_config_get_keyboard(XklEngine * engine,
 		xkl_debug(0, "Could not get tmp names\n");
 	}
 
-#endif
 	return xkb;
 }
 #else				/* no XKB headers */
@@ -446,19 +494,19 @@ xkl_xkb_activate_config_rec(XklEngine * engine, const XklConfigRec * data)
 #if 0
 	{
 		int i;
-		XklDebug(150, "New model: [%s]\n", data->model);
-		XklDebug(150, "New layouts: %p\n", data->layouts);
-		for (i = data->numLayouts; --i >= 0;)
-			XklDebug(150, "New layout[%d]: [%s]\n", i,
-				 data->layouts[i]);
-		XklDebug(150, "New variants: %p\n", data->variants);
-		for (i = data->numVariants; --i >= 0;)
-			XklDebug(150, "New variant[%d]: [%s]\n", i,
-				 data->variants[i]);
-		XklDebug(150, "New options: %p\n", data->options);
-		for (i = data->numOptions; --i >= 0;)
-			XklDebug(150, "New option[%d]: [%s]\n", i,
-				 data->options[i]);
+		xkl_debug(150, "New model: [%s]\n", data->model);
+		xkl_debug(150, "New layouts: %p\n", data->layouts);
+		for (i = 0; i < g_strv_length(data->layouts); i++)
+			xkl_debug(150, "New layout[%d]: [%s]\n", i,
+				  data->layouts[i]);
+		xkl_debug(150, "New variants: %p\n", data->variants);
+		for (i = 0; i < g_strv_length(data->variants); i++)
+			xkl_debug(150, "New variant[%d]: [%s]\n", i,
+				  data->variants[i]);
+		xkl_debug(150, "New options: %p\n", data->options);
+		for (i = 0; i < g_strv_length(data->options); i++)
+			xkl_debug(150, "New option[%d]: [%s]\n", i,
+				  data->options[i]);
 	}
 #endif
 
