@@ -34,6 +34,17 @@ enum { WM_NAME,
 	TOTAL_ATOMS
 };
 
+#define XKL_NUMBER_OF_REGISTRY_DOCS 2
+
+/* We have 3 values in XklEngineListenModes */
+enum {
+	XKLL_MANAGE_WINDOW_STATES_OFFSET = 0,
+	XKLL_TRACK_KEYBOARD_STATE_OFFSET,
+	XKLL_MANAGE_LAYOUTS_OFFSET,
+	XKLL_NUMBER_OF_LISTEN_MODES
+};
+
+
 struct _XklEnginePrivate {
 
 	gboolean group_per_toplevel_window;
@@ -42,9 +53,15 @@ struct _XklEnginePrivate {
 
 	gboolean skip_one_restore;
 
+	gboolean skip_one_save;
+
 	gint default_group;
 
-	guint listener_type;
+	/*
+	 * Separate counter for each of XklEngineListenModes!
+	 */
+	guchar listener_type_counters[XKLL_NUMBER_OF_LISTEN_MODES];
+#define xkl_engine_is_listening_for(engine, type) (xkl_engine_priv((engine), listener_type_counters)[type##_OFFSET] > 0)
 
 	guint secondary_groups_mask;
 
@@ -60,7 +77,7 @@ struct _XklEnginePrivate {
 
 	XklState curr_state;
 
-	gboolean criticalSection;
+	gboolean critical_section;
 
 	Atom atoms[TOTAL_ATOMS];
 
@@ -97,7 +114,8 @@ struct _XklEnginePrivate {
 	 * xkb: loads xml from XKB_BASE+"/rules/"+ruleset+".xml"
 	 * xmodmap: loads xml from XMODMAP_BASE+"/"+ruleset+".xml"
 	 */
-	 gboolean(*load_config_registry) (XklConfigRegistry * config);
+	 gboolean(*load_config_registry) (XklConfigRegistry * config,
+					  gboolean if_extras_needed);
 
 	/*
 	 * Write the configuration into the file (binary/textual)
@@ -153,6 +171,13 @@ struct _XklEnginePrivate {
 	 */
 	 gint(*process_x_event) (XklEngine * engine, XEvent * xev);
 
+	/*
+	 * Handles X errors.
+	 * return 0 if further processing is needed
+	 *        1 if error was handled
+	 */
+	 gint(*process_x_error) (XklEngine * engine, XErrorEvent * xerev);
+
 	/*
 	 * Flushes the cached server config info.
 	 * xkb: frees XkbDesc
@@ -245,9 +270,8 @@ extern XklEngine *xkl_get_the_engine(void);
 struct _XklConfigRegistryPrivate {
 	XklEngine *engine;
 
-	xmlDocPtr doc;
-
-	xmlXPathContextPtr xpath_context;
+	xmlDocPtr docs[XKL_NUMBER_OF_REGISTRY_DOCS];
+	xmlXPathContextPtr xpath_contexts[XKL_NUMBER_OF_REGISTRY_DOCS];
 };
 
 extern void xkl_engine_ensure_vtable_inited(XklEngine * engine);
@@ -261,7 +285,7 @@ extern void xkl_engine_process_property_evt(XklEngine * engine,
 extern void xkl_engine_process_create_window_evt(XklEngine * engine,
 						 XCreateWindowEvent * cev);
 
-extern void xkl_process_error(Display * dpy, XErrorEvent * evt);
+extern int xkl_process_error(Display * dpy, XErrorEvent * evt);
 
 extern void xkl_engine_process_state_modification(XklEngine * engine,
 						  XklEngineStateChange
@@ -388,17 +412,18 @@ extern void xkl_engine_one_switch_to_secondary_group_performed(XklEngine *
 
 extern gboolean xkl_config_registry_load_from_file(XklConfigRegistry *
 						   config,
-						   const gchar *
-						   file_name);
+						   const gchar * file_name,
+						   gint docidx);
 
 extern void xkl_config_registry_free(XklConfigRegistry * config);
 
 extern gchar *xkl_locale_from_utf8(XklConfigRegistry * config,
 				   const gchar * utf8string);
 
-extern gboolean xkl_config_registry_load_helper(XklConfigRegistry * config,
-					 const char default_ruleset[],
-					 const char base_dir[]);
+extern gboolean xkl_config_registry_load_helper(XklConfigRegistry * config, const char
+						default_ruleset[],
+						const char base_dir[],
+						gboolean if_extras_needed);
 
 #define XKLAVIER_STATE_PROP_LENGTH 2
 
@@ -412,9 +437,49 @@ extern gboolean xkl_config_registry_load_helper(XklConfigRegistry * config,
 #define xkl_engine_get_display(engine) (xkl_engine_priv(engine,display))
 #define xkl_engine_vcall(engine,func)  (*(engine)->priv->func)
 
+#define xkl_config_registry_is_initialized(config) \
+  ( xkl_config_registry_priv(config,xpath_contexts[0]) != NULL )
+
 #define xkl_config_registry_priv(config,member)  (config)->priv->member
 #define xkl_config_registry_get_engine(config) ((config)->priv->engine)
 
+#define XKBCR_MODEL_PATH "/xkbConfigRegistry/modelList/model"
+#define XKBCR_LAYOUT_PATH "/xkbConfigRegistry/layoutList/layout"
+#define XKBCR_VARIANT_PATH XKBCR_LAYOUT_PATH "/variantList/variant"
+#define XKBCR_GROUP_PATH "/xkbConfigRegistry/optionList/group"
+#define XKBCR_OPTION_PATH XKBCR_GROUP_PATH "/option"
+
+#define XML_TAG_DESCR "description"
+#define XML_TAG_SHORT_DESCR "shortDescription"
+#define XML_TAG_VENDOR "vendor"
+#define XML_TAG_COUNTRY_LIST "countryList"
+#define XML_TAG_LANGUAGE_LIST "languageList"
+#define XML_TAG_ISO3166ID "iso3166Id"
+#define XML_TAG_ISO639ID "iso639Id"
+
+extern void
+
+
+
+
+
+xkl_config_registry_foreach_in_xpath_with_param(XklConfigRegistry * config,
+						const gchar * format,
+						const gchar * value,
+						XklConfigItemProcessFunc func,
+						gpointer data);
+
+extern void xkl_config_registry_foreach_in_xpath(XklConfigRegistry *
+						 config,
+						 xmlXPathCompExprPtr
+						 xpath_comp_expr,
+						 XklConfigItemProcessFunc
+						 func, gpointer data);
+
+extern gboolean xkl_read_config_item(XklConfigRegistry * config,
+				     gint doc_index, xmlNodePtr iptr,
+				     XklConfigItem * item);
+
 extern gint xkl_debug_level;
 
 extern const gchar *xkl_last_error_message;
